const fs = require('fs');
const path = require('path')
const { readFile, writeFile } = require('./utils')
class ProfileHandler {
  get folderPath() {
    return path.resolve(global.hpm.cwd, '.product_profile');
  }
  /**
   *
   * @param {object} data
   * @param {boolean} type  true:新建  false:重新配置
   */

  getProfile() {
    if (!fs.existsSync(this.folderPath)) {
      return []
    }
    const data = fs.readdirSync(this.folderPath);
    if (!data.length) {
      return []
    }
    return data
      .filter(file => file.endsWith('.json'))
      .map(file => ({
        fileName: path.parse(file).name,
        birthtime: fs.statSync(path.resolve(this.folderPath, file)).birthtimeMs
      }))
      .sort((a, b) => {
        return b.birthtime - a.birthtime;
      })
  }

  openProfile(name) {
    const profileName = name.endsWith('.json') ? name : `${name}.json`
    const profilePath = path.resolve(this.folderPath, profileName)
    if (!fs.existsSync(profilePath)) {
      return []
    }
    return readFile(profilePath)
  }

  deleteProfile({ name, version }) {
    const profilePath = path.resolve(this.folderPath, `${name}_${version}.json`)
    if (!fs.existsSync(profilePath)) {
      throw new Error('file is not exist')
    }
    fs.unlinkSync(profilePath)
  }

  createOrUpdateProfile(data, type = 'created') {
    if (!fs.existsSync(this.folderPath)) {
      fs.mkdirSync(this.folderPath)
    }
    const currentFile = this.isNameExist(data.profile.name)
    if (currentFile) {
      const fileName = path.parse(currentFile.name).name
      const profileInfo = {
        name: fileName.substr(0, fileName.lastIndexOf('_')),
        version: fileName.substr(fileName.lastIndexOf('_') + 1)
      }
      if (type === 'created') {
        //  新建的时候重名提示用户
        throw new Error('name is exist')
      } else if (type === 'updated') {
        //  修改配置时重名覆盖原文件，删除原文件重新创建
        this.deleteProfile(profileInfo)
      }
    }
    const profileName = `${data.profile.name}_${data.profile.version}` || 'product'
    return writeFile(path.resolve(this.folderPath, `${profileName}.json`), JSON.stringify(data, null, 4))
  }

  isNameExist(name) {
    const data = fs.readdirSync(this.folderPath, { withFileTypes: true });
    return data.find(file =>
      file.name === `${name}_${file.name.substr(file.name.lastIndexOf('_') + 1)}`
    )
  }
}

module.exports = ProfileHandler
